/*************************************************************************
Title........ duart.h
Date......... Tue Dec 14 11:24:36  1999
Author....... J.M.Tarrant
*************************************************************************/

/****	Defines */


	#define	DUART0BASE	(void*)(FPGA_BASE  + 0x00400000)
	#define	DUART1BASE	(void*)(FPGA_BASE  + 0x00400010)
	#define	DUART2BASE	(void*)(FPGA_BASE  + 0x00400020)

/* Mode register 1 bits */
	#define	PARITYEVEN	0x00
	#define	PARITYODD	0x04
	#define	PARITYNONE	0x10
	#define	PARITYBITS	0x1c

	#define	BITS6		0x01
	#define	BITS7		0x02
	#define	BITS8		0x03
	#define	CHARBITS	0x03

	#define RXRTS		0x80
	#define RXNONE		0x00
	#define RXBITS		0x80

/* Mode register 2 bits */
	#define	STOP1		0x07
	#define	STOP2		0x0f
	#define	STOPBITS	0x0f

	#define	TXRTS		0x20
	#define	TXCTS		0x10
	#define	TXNONE		0x00
	#define	TXBITS		0x30

/* Auxiliary control register */
	#define	BAUDSET1	0x00
	#define	BAUDSET2	0x80
	#define	BAUDSETBITS	0x80

/* Baud select */
	#define	BAUD50		0
	#define	BAUD75		1
	#define	BAUD110		2
	#define	BAUD134		3
	#define	BAUD150		4
	#define	BAUD200		5
	#define	BAUD300		6
	#define	BAUD600		7
	#define	BAUD1050	8
	#define	BAUD1200	9
	#define BAUD1800	10
	#define	BAUD2000	11
	#define	BAUD2400	12
	#define	BAUD4800	13
	#define	BAUD7200	14
	#define	BAUD9600	15
	#define	BAUD19200	16
	#define	BAUD38400	17
	#define	BAUD76800	18


/*	clock select register */
	#define	RXBAUDBITS	0xf0
	#define	TXBAUDBITS	0x0f

/****	Command register */

	#define	TX_DISBALE_CMD	0x08
	#define	TX_ENABLE_CMD	0x04
	#define	TX_RESET_CMD	0x30

	#define	RX_DISABLE_CMD	0x02
	#define	RX_ENABLE_CMD	0x01
	#define	RX_RESET_CMD	0x20

	#define	ERR_RESET_CMD	0x40
	#define	MR_RESET_CMD	0x10

/*	Interrupt status register */
	#define	TXRDYA			0x01
	#define	RXRDYA			0x02
	#define	DBA				0x04
	#define	XTAL_RDY		0x08
	#define	TXRDYB			0x10
	#define	RXRDYB			0x20
	#define	DBB				0x40
	#define	COS				0x80

/*	Status register */
	#define	SR_RB			0x80
	#define	SR_FE			0x40
	#define	SR_PE			0x20
	#define	SR_OE			0x10

	#define	ERROR_BITS	(SR_RB+SR_FE+SR_PE+SR_OE)


/****	Type Definitions */

typedef	struct reg68681wr
	{
	volatile	UBYTE	mra;
	volatile	UBYTE	csra;
	volatile	UBYTE	cra;
	volatile	UBYTE	tba;
	volatile	UBYTE	acr;
	volatile	UBYTE	ier;
	volatile	UBYTE	ctur;
	volatile	UBYTE	ctlr;

	volatile	UBYTE	mrb;
	volatile	UBYTE	csrb;
	volatile	UBYTE	crb;
	volatile	UBYTE	tbb;
	volatile	UBYTE	ivr;
	volatile	UBYTE	opcr;
	volatile	UBYTE	bset;
	volatile	UBYTE	bclr;
	} REG68681W;

typedef	struct reg68681r
	{
	volatile	UBYTE	mra;
	volatile	UBYTE	sra;
	volatile	UBYTE	csra;
	volatile	UBYTE	rba;
	volatile	UBYTE	ipcr;
	volatile	UBYTE	isr;
	volatile	UBYTE	ctru;
	volatile	UBYTE	ctrl;

	volatile	UBYTE	mrb;
	volatile	UBYTE	srb;
	volatile	UBYTE	csrb;
	volatile	UBYTE	rbb;
	volatile	UBYTE	ivr;
	volatile	UBYTE	ip;
	volatile	UBYTE	strctr;
	volatile	UBYTE	stpctr;
	} REG68681R;

typedef	union	reg68681
	{
	REG68681R	read;
	REG68681W	write;
	} REG68681;

typedef	struct ram68681w
	{
	volatile	UBYTE	mr1a;
	volatile	UBYTE	mr2a;
	volatile	UBYTE	csra;

	volatile	UBYTE	mr1b;
	volatile	UBYTE	mr2b;
	volatile	UBYTE	csrb;

	volatile	UBYTE	acr;
	volatile	UBYTE	ier;
	} RAM68681W;

typedef	struct	duartcfg
	{
	void*		base;
	void		(*isr)();
	BYTE*		txabuffer;
	BYTE*		rxabuffer;
	BYTE*		txbbuffer;
	BYTE*		rxbbuffer;
	BYTE		txamask;
	BYTE		rxamask;
	BYTE		txbmask;
	BYTE		rxbmask;

	BYTE		txbauda;
	BYTE		rxbauda;
	BYTE		bitsa;
	BYTE		stopa;
	BYTE		paritya;
	BYTE		txhsa;
	BYTE		rxhsa;

	BYTE		txbaudb;
	BYTE		rxbaudb;
	BYTE		bitsb;
	BYTE		stopb;
	BYTE		parityb;
	BYTE		txhsb;
	BYTE		rxhsb;
	} DUARTCFG;


typedef	struct	buffer
	{
	UBYTE	txin;
	UBYTE	txout;
	UBYTE	txmask;
	UBYTE	rxin;
	UBYTE	rxout;
	UBYTE	rxmask;
	UBYTE*	txbuffer;
	UBYTE*	rxbuffer;
	} SERIALBUFFER;

typedef	struct	duartvars
	{
	const DUARTCFG*	config;
	REG68681*		base;
	RAM68681W		regs;
	SERIALBUFFER	cha;
	SERIALBUFFER	chb;
	} DUARTVARS;


typedef	struct duartservices
	{
	BYTE	(*configure)(const DEVICE* dev,BYTE txbaud,BYTE rxbaud,BYTE data,BYTE stop,BYTE parity,BYTE txhs ,BYTE rxhs);
	BYTE	(*tx)(const DEVICE* dev,char ch);
	WORD	(*rx)();
	BYTE 	(*txempty)();
	BYTE 	(*rxempty)();
	void 	(*cleartx)();
	void 	(*clearrx)();
	} DUARTSERVICES;


/****	Prototypes */

	#include	"duart.p"

/*************************************************************************/
