/*************************************************************************
Title........ uart.h
Date......... 2 Jan 2003
Author....... Alan Austin
*************************************************************************/


/* UART Register Addresses*/
#define	CALYPSO_UART_BASE		0x3f80000

/* UART Register Addresses*/

// Transmit data - Write Only
#define CALYPSO_UART_TXDATA		CALYPSO_UART_BASE

// Receive Data - Read Only
#define CALYPSO_UART_RXDATA		CALYPSO_UART_BASE

// Control Register - Write Only
#define CALYPSO_UART_CONTROL	CALYPSO_UART_BASE+0x02

// Status Register - Read Only
#define CALYPSO_UART_STATUS		CALYPSO_UART_BASE+0x02



/* Control Register Bit Defines

D7	RX Interrupt Enable	(1=Enable)
D6	TX Interrupt Enable	(1=Enable)
D5	BAUD1
D4	BAUD2
D3	RTS	(1=active)
D2
D1	Clear RX Ready (1=clear)
D0

Baud	BAUD1	BAUD2
1200	0		0
2400	0		1
4800	1		0
9600	1		1

*/
#define RXINT_ENABLE 	0x80
#define TXINT_ENABLE 	0x40
#define RTS				0x08
#define CLEAR_RXRDY		0x02



/* Status Register Bit Defines

D7	RX Interrupt Enable	Status
D6	TX Interrupt Enable	Status
D5
D4	CTS	(1=active)
D3
D2	Frame Error
D1	RX Ready (1=Character received)
D0	TX Ready (1=Character sent)


*/
#define RXINT_ENABLE_STATUS 	0x80
#define TXINT_ENABLE_STATUS 	0x40
#define CTS						0x10
#define FRAME_ERROR				0x04
#define RXRDY					0x02
#define TXRDY					0x01





/****	Type Definitions */


typedef	struct	uartcfg
	{
	BYTE		txbaud;
	BYTE		rxbaud;
	BYTE		bits;
	BYTE		stop;
	BYTE		parity;
	BYTE		txhs;
	BYTE		rxhs;
	} UARTCFG;



typedef	struct	uartvars
	{
	WORD	txin;
	WORD	txout;
	WORD	rxin;
	WORD	rxout;
	UBYTE	state;
	} UARTVARS;



typedef	struct uartservices
	{
	BYTE	(*configure)(const DEVICE* dev,BYTE txbaud,BYTE rxbaud,BYTE data,BYTE stop,BYTE parity,BYTE txhs ,BYTE rxhs);
	BYTE	(*tx)(const DEVICE* dev,char ch);
	WORD	(*rx)();
	BYTE 	(*txempty)();
	BYTE 	(*rxempty)();
	void 	(*cleartx)();
	void 	(*clearrx)();
	} UARTSERVICES;



/****	Prototypes */

	#include	"uart.p"

/*************************************************************************/
